@@ -455,12 +455,34 @@ func (o *snapshotter) getCleanupDirectories(ctx context.Context, t storage.Trans
 		return nil, err
 	}
 
+	remoteSnapshotNames := make(map[string]struct{})
+	{
+		keyToID := make(map[string]string, len(ids))
+		for id, key := range ids {
+			keyToID[key] = id
+		}
+		if err := storage.WalkInfo(ctx, func(ctx context.Context, info snapshots.Info) error {
+			if _, ok := info.Labels[remoteLabel]; ok {
+				if id, exists := keyToID[info.Name]; exists {
+					remoteSnapshotNames[id] = struct{}{}
+				}
+			}
+			return nil
+		}); err != nil {
+			return nil, err
+		}
+	}
+
 	cleanup := []string{}
 	for _, d := range dirs {
 		if !cleanupCommitted {
 			if _, ok := ids[d]; ok {
 				continue
 			}
+		} else {
+			if _, ok := remoteSnapshotNames[d]; !ok {
+				continue
+			}
 		}
 
 		cleanup = append(cleanup, filepath.Join(snapshotDir, d))
@@ -760,10 +782,13 @@ func (o *snapshotter) restoreRemoteSnapshot(ctx context.Context) error {
 			if err != nil {
 				return err
 			}
-			if err := os.Mkdir(filepath.Join(o.root, "snapshots", id), 0700); err != nil {
+			if err := os.Mkdir(filepath.Join(o.root, "snapshots", id), 0700); err != nil && !os.IsExist(err) {
 				return err
 			}
-			return os.Mkdir(o.upperPath(id), 0755)
+			if err := os.Mkdir(o.upperPath(id), 0755); err != nil && !os.IsExist(err) {
+				return err
+			}
+			return nil
 		}(); err != nil {
 			return fmt.Errorf("failed to create remote snapshot directory: %s: %w", info.Name, err)
 		}
